// Copyright The Prometheus Authors
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//go:build !windows

package procfs

import (
	"testing"
)

func TestKernelHung(t *testing.T) {
	fs, err := NewFS(procfsFixtures)
	if err != nil {
		t.Fatalf("failed to access %s: %v", procfsFixtures, err)
	}

	hung, err := fs.KernelHung()
	if err != nil {
		t.Fatalf("failed to collect %s/sys/kernel/hung_task_detect_count: %v", procfsFixtures, err)
	}

	if *hung.HungTaskDetectCount != 6 {
		t.Errorf("hung_task_detect_count, want %d got %d", 6, *hung.HungTaskDetectCount)
	}
}
